@@ -159,6 +159,11 @@ static int data_recv(uint8_t *resp, int length, void *data)
 
 	/* Add data to tag mem */
 	tagdata = near_tag_get_data(t1_tag->tag, &data_length);
+
+	/* Check that we have enough free space */
+	if (data_length - t1_tag->data_read < (uint)length)
+		return -EINVAL;
+
 	memcpy(tagdata + t1_tag->data_read, resp + 1, length);
 
 	/* Next segment */
@@ -316,6 +321,11 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 		DBG("READ Static complete");
 
 		tagdata = near_tag_get_data(t1_tag->tag, &data_length);
+
+		/* Check that we have enough free space */
+		if (data_length < (size_t)TAG_T1_DATA_LENGTH(cc))
+			return -EINVAL;
+
 		memcpy(tagdata, cc + LEN_CC_BYTES, TAG_T1_DATA_LENGTH(cc));
 
 		near_tag_set_memory_layout(tag, NEAR_TAG_MEMORY_STATIC);
@@ -629,11 +639,6 @@ static int nfctype1_write(uint32_t adapter_idx, uint32_t target_idx,
 
 	return data_write(adapter_idx, target_idx, ndef, cb);
 
-	if (err < 0)
-		goto out_err;
-
-	return 0;
-
 out_err:
 	if (cb)
 		cb(adapter_idx, target_idx, err);
