@@ -142,24 +142,14 @@ static char *rf_mode_to_string(struct near_adapter *adapter)
 
 static void polling_changed(struct near_adapter *adapter)
 {
-	dbus_bool_t polling;
-
-	polling = adapter->polling;
-	near_dbus_property_changed_basic(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Polling",
-					DBUS_TYPE_BOOLEAN, &polling);
+	g_dbus_emit_property_changed(connection, adapter->path,
+					NFC_ADAPTER_INTERFACE, "Polling");
 }
 
 static void rf_mode_changed(struct near_adapter *adapter)
 {
-	const char *rf_mode = rf_mode_to_string(adapter);
-
-	if (!rf_mode)
-		return;
-
-	near_dbus_property_changed_basic(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Mode",
-					DBUS_TYPE_STRING, &rf_mode);
+	g_dbus_emit_property_changed(connection, adapter->path,
+					NFC_ADAPTER_INTERFACE, "Mode");
 }
 
 static int adapter_start_poll(struct near_adapter *adapter)
@@ -171,14 +161,12 @@ static int adapter_start_poll(struct near_adapter *adapter)
 		DBG("Clearing tags");
 
 		g_hash_table_remove_all(adapter->tags);
-		__near_adapter_tags_changed(adapter->idx);
 	}
 
 	if (g_hash_table_size(adapter->devices) > 0) {
 		DBG("Clearing devices");
 
 		g_hash_table_remove_all(adapter->devices);
-		__near_adapter_devices_changed(adapter->idx);
 	}
 
 	DBG("Poll mode 0x%x", adapter->poll_mode);
@@ -202,28 +190,102 @@ static int adapter_start_poll(struct near_adapter *adapter)
 	return 0;
 }
 
-static void append_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct near_adapter *adapter = value;
-	DBusMessageIter *iter = user_data;
-
-	DBG("%s", adapter->path);
-
-	if (!adapter->path)
-		return;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-							&adapter->path);
-}
-
-void __near_adapter_list(DBusMessageIter *iter, void *user_data)
-{
-	g_hash_table_foreach(adapter_hash, append_path, iter);
-}
-
-static void append_protocols(DBusMessageIter *iter, void *user_data)
+static gboolean property_get_mode(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
 {
 	struct near_adapter *adapter = user_data;
+	const char *rf_mode;
+
+	rf_mode = rf_mode_to_string(adapter);
+	if (!rf_mode)
+		return FALSE;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &rf_mode);
+
+	return TRUE;
+}
+
+static gboolean property_get_polling(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_adapter *adapter = user_data;
+	dbus_bool_t val;
+
+	val = adapter->polling;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_BOOLEAN, &val);
+
+	return TRUE;
+}
+
+static gboolean property_get_powered(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_adapter *adapter = user_data;
+	dbus_bool_t val;
+
+	val = adapter->powered;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_BOOLEAN, &val);
+
+	return TRUE;
+}
+
+static void set_powered(GDBusPendingPropertySet id, dbus_bool_t powered,
+								void *data)
+{
+	struct near_adapter *adapter = data;
+	int err;
+
+	err = __near_netlink_adapter_enable(adapter->idx, powered);
+	if (err < 0) {
+		if (err == -EALREADY) {
+			if (powered)
+				g_dbus_pending_property_error(id,
+						NFC_ERROR_INTERFACE ".Failed",
+						"Device already enabled");
+			else
+				g_dbus_pending_property_error(id,
+						NFC_ERROR_INTERFACE ".Failed",
+						"Device already disabled");
+		}
+
+		g_dbus_pending_property_error(id,
+						NFC_ERROR_INTERFACE ".Failed",
+						strerror(err));
+
+		return;
+	}
+
+	g_dbus_pending_property_success(id);
+
+	adapter->powered = powered;
+
+	g_dbus_emit_property_changed(connection, adapter->path,
+					NFC_ADAPTER_INTERFACE, "Powered");
+}
+
+static void property_set_powered(const GDBusPropertyTable *property,
+					DBusMessageIter *value,
+					GDBusPendingPropertySet id, void *data)
+{
+	dbus_bool_t powered;
+
+	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_BOOLEAN) {
+		g_dbus_pending_property_error(id,
+					NFC_ERROR_INTERFACE ".InvalidArguments",
+					"Invalid arguments in method call");
+		return;
+	}
+
+	dbus_message_iter_get_basic(value, &powered);
+
+	set_powered(id, powered, data);
+}
+
+static void append_protocols(DBusMessageIter *iter,
+					struct near_adapter *adapter)
+{
 	const char *str;
 
 	DBG("protocols 0x%x", adapter->protocols);
@@ -259,180 +321,20 @@ static void append_protocols(DBusMessageIter *iter, void *user_data)
 	}
 }
 
-static void append_tag_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct near_tag *tag = value;
-	DBusMessageIter *iter = user_data;
-	const char *tag_path;
-
-	tag_path = __near_tag_get_path(tag);
-	if (!tag_path)
-		return;
-
-	DBG("%s", tag_path);
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH, &tag_path);
-}
-
-static void append_tags(DBusMessageIter *iter, void *user_data)
+static gboolean property_get_protocols(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
 {
 	struct near_adapter *adapter = user_data;
+	DBusMessageIter dict;
 
-	DBG("");
+	dbus_message_iter_open_container(iter, DBUS_TYPE_ARRAY,
+						DBUS_TYPE_STRING_AS_STRING, &dict);
 
-	g_hash_table_foreach(adapter->tags, append_tag_path, iter);
-}
+	append_protocols(&dict, adapter);
 
-static void append_device_path(gpointer key, gpointer value, gpointer user_data)
-{
-	struct near_device *device = value;
-	DBusMessageIter *iter = user_data;
-	const char *device_path;
+	dbus_message_iter_close_container(iter, &dict);
 
-	device_path = __near_device_get_path(device);
-	if (!device_path)
-		return;
-
-	DBG("%s", device_path);
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-							&device_path);
-}
-
-static void append_devices(DBusMessageIter *iter, void *user_data)
-{
-	struct near_adapter *adapter = user_data;
-
-	DBG("");
-
-	g_hash_table_foreach(adapter->devices, append_device_path, iter);
-}
-
-void __near_adapter_tags_changed(uint32_t adapter_idx)
-{
-	struct near_adapter *adapter;
-
-	DBG("");
-
-	adapter = g_hash_table_lookup(adapter_hash,
-					GINT_TO_POINTER(adapter_idx));
-	if (!adapter)
-		return;
-
-	near_dbus_property_changed_array(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Tags",
-					DBUS_TYPE_OBJECT_PATH, append_tags,
-					adapter);
-}
-
-void __near_adapter_devices_changed(uint32_t adapter_idx)
-{
-	struct near_adapter *adapter;
-
-	DBG("");
-
-	adapter = g_hash_table_lookup(adapter_hash,
-					GINT_TO_POINTER(adapter_idx));
-	if (!adapter)
-		return;
-
-	near_dbus_property_changed_array(adapter->path,
-					NFC_ADAPTER_INTERFACE, "Devices",
-					DBUS_TYPE_OBJECT_PATH, append_devices,
-					adapter);
-}
-
-static DBusMessage *get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct near_adapter *adapter = data;
-	const char *rf_mode;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	dbus_bool_t val;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (!reply)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	near_dbus_dict_open(&array, &dict);
-
-	val = adapter->powered;
-	near_dbus_dict_append_basic(&dict, "Powered",
-					DBUS_TYPE_BOOLEAN, &val);
-
-	val = adapter->polling;
-	near_dbus_dict_append_basic(&dict, "Polling",
-				    DBUS_TYPE_BOOLEAN, &val);
-
-	rf_mode = rf_mode_to_string(adapter);
-	if (rf_mode)
-		near_dbus_dict_append_basic(&dict, "Mode",
-						DBUS_TYPE_STRING, &rf_mode);
-
-	near_dbus_dict_append_array(&dict, "Protocols",
-				DBUS_TYPE_STRING, append_protocols, adapter);
-
-	near_dbus_dict_append_array(&dict, "Tags",
-				DBUS_TYPE_OBJECT_PATH, append_tags, adapter);
-
-	near_dbus_dict_append_array(&dict, "Devices",
-				DBUS_TYPE_OBJECT_PATH, append_devices, adapter);
-
-	near_dbus_dict_close(&array, &dict);
-
-	return reply;
-}
-
-static DBusMessage *set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct near_adapter *adapter = data;
-	DBusMessageIter iter, value;
-	const char *name;
-	int type, err;
-
-	DBG("conn %p", conn);
-
-	if (!dbus_message_iter_init(msg, &iter))
-		return __near_error_invalid_arguments(msg);
-
-	dbus_message_iter_get_basic(&iter, &name);
-	dbus_message_iter_next(&iter);
-	dbus_message_iter_recurse(&iter, &value);
-
-	type = dbus_message_iter_get_arg_type(&value);
-
-	if (g_str_equal(name, "Powered")) {
-		dbus_bool_t powered;
-
-		if (type != DBUS_TYPE_BOOLEAN)
-			return __near_error_invalid_arguments(msg);
-
-		dbus_message_iter_get_basic(&value, &powered);
-
-		err = __near_netlink_adapter_enable(adapter->idx, powered);
-		if (err < 0) {
-			if (err == -EALREADY) {
-				if (powered)
-					return __near_error_already_enabled(msg);
-				else
-					return __near_error_already_disabled(msg);
-			}
-
-			return __near_error_failed(msg, -err);
-		}
-
-		adapter->powered = powered;
-	} else {
-		return __near_error_invalid_property(msg);
-	}
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+	return TRUE;
 }
 
 static DBusMessage *start_poll_loop(DBusConnection *conn,
@@ -444,6 +346,11 @@ static DBusMessage *start_poll_loop(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (!adapter->powered) {
+		near_error("Adapter is down, can not start polling");
+		return __near_error_failed(msg, ENODEV);
+	}
+
 	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &dbus_mode,
 							DBUS_TYPE_INVALID);
 
@@ -596,23 +503,18 @@ void __near_adapter_stop_check_presence(uint32_t adapter_idx,
 }
 
 static const GDBusMethodTable adapter_methods[] = {
-	{ GDBUS_METHOD("GetProperties",
-				NULL, GDBUS_ARGS({"properties", "a{sv}"}),
-				get_properties) },
-	{ GDBUS_METHOD("SetProperty",
-				GDBUS_ARGS({"name", "s"}, {"value", "v"}),
-				NULL, set_property) },
 	{ GDBUS_METHOD("StartPollLoop", GDBUS_ARGS({"name", "s"}), NULL,
 							start_poll_loop) },
 	{ GDBUS_METHOD("StopPollLoop", NULL, NULL, stop_poll_loop) },
 	{ },
 };
 
-static const GDBusSignalTable adapter_signals[] = {
-	{ GDBUS_SIGNAL("PropertyChanged",
-				GDBUS_ARGS({"name", "s"}, {"value", "v"})) },
-	{ GDBUS_SIGNAL("TagFound", GDBUS_ARGS({"address", "o"})) },
-	{ GDBUS_SIGNAL("TagLost", GDBUS_ARGS({"address", "o"})) },
+static const GDBusPropertyTable adapter_properties[] = {
+	{ "Mode", "s", property_get_mode },
+	{ "Powered", "b", property_get_powered, property_set_powered },
+	{ "Polling", "b", property_get_polling },
+	{ "Protocols", "as", property_get_protocols },
+
 	{ }
 };
 
@@ -698,7 +600,8 @@ int __near_adapter_set_dep_state(uint32_t idx, bool dep)
 		if (adapter->dep_timer > 0)
 			g_source_remove(adapter->dep_timer);
 
-		__near_adapter_devices_changed(idx);
+		if (!__near_device_register_interface(adapter->device_link))
+			return -ENODEV;
 	}
 
 	return 0;
@@ -732,8 +635,8 @@ int __near_adapter_add(struct near_adapter *adapter)
 
 	g_dbus_register_interface(connection, adapter->path,
 					NFC_ADAPTER_INTERFACE,
-					adapter_methods, adapter_signals,
-					NULL, adapter, NULL);
+					adapter_methods, NULL,
+					adapter_properties, adapter, NULL);
 
 	return 0;
 }
@@ -767,8 +670,6 @@ static void tag_read_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
 		return;
 	}
 
-	__near_adapter_tags_changed(adapter_idx);
-
 	adapter->presence_timeout =
 		g_timeout_add_seconds(CHECK_PRESENCE_PERIOD,
 					check_presence, adapter);
@@ -842,6 +743,8 @@ static int adapter_add_device(struct near_adapter *adapter,
 	struct near_device *device;
 	int err;
 
+	DBG();
+
 	device = __near_device_add(adapter->idx, target_idx, nfcid, nfcid_len);
 	if (!device)
 		return -ENODEV;
@@ -858,8 +761,12 @@ static int adapter_add_device(struct near_adapter *adapter,
 
 	adapter->device_link = device;
 
-	if (adapter->dep_up)
+	if (adapter->dep_up) {
+		if (!__near_device_register_interface(device))
+			return -ENODEV;
+
 		return 0;
+	}
 
 	err = __near_netlink_dep_link_up(adapter->idx, target_idx,
 					NFC_COMM_ACTIVE, NFC_RF_INITIATOR);
@@ -919,19 +826,56 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
 	rf_mode_changed(adapter);
 
-	if (g_hash_table_remove(adapter->tags,
-			GINT_TO_POINTER(target_idx))) {
-		__near_adapter_tags_changed(idx);
-
+	if (g_hash_table_remove(adapter->tags, GINT_TO_POINTER(target_idx)))
 		return 0;
+
+	if (g_hash_table_remove(adapter->devices, GINT_TO_POINTER(target_idx)))
+		return 0;
+
+	return 0;
+}
+
+static gboolean poll_error(gpointer user_data)
+{
+	struct near_adapter *adapter = user_data;
+	bool reset;
+
+	DBG("adapter %d", adapter->idx);
+
+	reset = near_setting_get_bool("ResetOnError");
+	if (reset) {
+		near_error("Resetting nfc%d", adapter->idx);
+		 __near_netlink_adapter_enable(adapter->idx, false);
+		 __near_netlink_adapter_enable(adapter->idx, true);
 	}
 
-	if (g_hash_table_remove(adapter->devices,
-			GINT_TO_POINTER(target_idx))) {
-		__near_adapter_devices_changed(idx);
+	adapter_start_poll(adapter);
 
+	return FALSE;
+}
+
+int __near_adapter_get_targets_done(uint32_t idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("idx %d", idx);
+
+	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
+	if (!adapter)
+		return -ENODEV;
+
+	if (g_hash_table_size(adapter->devices) > 0)
 		return 0;
-	}
+
+	if (g_hash_table_size(adapter->tags) > 0)
+		return 0;
+
+	near_error("No targets found - Polling error");
+
+	adapter->polling = false;
+	polling_changed(adapter);
+
+	g_idle_add(poll_error, adapter);
 
 	return 0;
 }
@@ -957,8 +901,6 @@ int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 	if (ret < 0)
 		return ret;
 
-	__near_adapter_devices_changed(idx);
-
 	return 0;
 }
 
@@ -978,7 +920,6 @@ int __near_adapter_remove_device(uint32_t idx)
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
 	rf_mode_changed(adapter);
-	__near_adapter_devices_changed(idx);
 
 	adapter->dep_up = false;
 
