@@ -68,6 +68,12 @@ struct nlnfc_state {
 static struct nlnfc_state *nfc_state;
 static GIOChannel *netlink_channel = NULL;
 
+struct send_msg_data {
+	void *data;
+	int *done;
+	int (*finish_handler)(struct nl_msg *, void *);
+};
+
 static int error_handler(struct sockaddr_nl *nla, struct nlmsgerr *err,
 			 void *arg)
 {
@@ -80,13 +86,17 @@ static int error_handler(struct sockaddr_nl *nla, struct nlmsgerr *err,
 	return NL_STOP;
 }
 
-static int finish_handler(struct nl_msg *msg, void *arg)
+static int __finish_handler(struct nl_msg *msg, void *arg)
 {
-	int *ret = arg;
+	struct send_msg_data *data = arg;
+
 
 	DBG("");
 
-	*ret = 1;
+	if (data->finish_handler)
+		data->finish_handler(msg, data->data);
+
+	*(data->done) = 1;
 
 	return NL_SKIP;
 }
@@ -102,17 +112,19 @@ static int ack_handler(struct nl_msg *msg, void *arg)
 	return NL_STOP;
 }
 
-static int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
+static int __nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
 			int (*rx_handler)(struct nl_msg *, void *),
+			int (*finish_handler)(struct nl_msg *, void *),
 			void *data)
 {
 	struct nl_cb *cb;
 	int err, done;
+	struct send_msg_data send_data;
 
 	DBG("");
 
 	cb = nl_cb_alloc(NL_CB_DEFAULT);
-	if (cb == NULL)
+	if (!cb)
 		return -ENOMEM;
 
 	err = nl_send_auto_complete(sock, msg);
@@ -124,12 +136,15 @@ static int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
 	}
 
 	err = done = 0;
+	send_data.done = &done;
+	send_data.data = data;
+	send_data.finish_handler = finish_handler;
 
+	nl_cb_set(cb, NL_CB_FINISH, NL_CB_CUSTOM, __finish_handler, &send_data);
 	nl_cb_err(cb, NL_CB_CUSTOM, error_handler, &err);
-	nl_cb_set(cb, NL_CB_FINISH, NL_CB_CUSTOM, finish_handler, &done);
 	nl_cb_set(cb, NL_CB_ACK, NL_CB_CUSTOM, ack_handler, &done);
 
-	if (rx_handler != NULL)
+	if (rx_handler)
 		nl_cb_set(cb, NL_CB_VALID, NL_CB_CUSTOM, rx_handler, data);
 
 	while (err == 0 && done == 0)
@@ -140,6 +155,12 @@ static int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
 	return err;
 }
 
+static inline int nl_send_msg(struct nl_sock *sock, struct nl_msg *msg,
+			int (*rx_handler)(struct nl_msg *, void *),
+			void *data)
+{
+	return __nl_send_msg(sock, msg, rx_handler, NULL, data);
+}
 
 static int get_devices_handler(struct nl_msg *n, void *arg)
 {
@@ -147,26 +168,25 @@ static int get_devices_handler(struct nl_msg *n, void *arg)
 	struct nlattr *attrs[NFC_ATTR_MAX + 1];
 	char *name;
 	uint32_t idx, protocols;
-	near_bool_t powered;
+	bool powered;
 
 	DBG("");
 
 	genlmsg_parse(nlh, 0, attrs, NFC_ATTR_MAX, NULL);
 
-	if (attrs[NFC_ATTR_DEVICE_INDEX] == NULL ||
-	    attrs[NFC_ATTR_DEVICE_NAME] == NULL ||
-	    attrs[NFC_ATTR_PROTOCOLS] == NULL) {
+	if (!attrs[NFC_ATTR_DEVICE_INDEX] ||
+	    !attrs[NFC_ATTR_DEVICE_NAME] ||
+	    !attrs[NFC_ATTR_PROTOCOLS]) {
 		nl_perror(NLE_MISSING_ATTR, "NFC_CMD_GET_DEVICE");
 		return NL_STOP;
 	}
 
-
 	idx = nla_get_u32(attrs[NFC_ATTR_DEVICE_INDEX]);
 	name = nla_get_string(attrs[NFC_ATTR_DEVICE_NAME]);
 	protocols = nla_get_u32(attrs[NFC_ATTR_PROTOCOLS]);
 
-	if (attrs[NFC_ATTR_DEVICE_POWERED] == NULL)
-		powered = FALSE;
+	if (!attrs[NFC_ATTR_DEVICE_POWERED])
+		powered = false;
 	else
 		powered = nla_get_u8(attrs[NFC_ATTR_DEVICE_POWERED]);
 
@@ -183,16 +203,16 @@ int __near_netlink_get_adapters(void)
 
 	DBG("");
 
-	if (nfc_state == NULL || nfc_state->nfc_id < 0)
+	if (!nfc_state || nfc_state->nfc_id < 0)
 		return -ENODEV;
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			  NLM_F_DUMP, NFC_CMD_GET_DEVICE, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto out;
 	}
@@ -215,12 +235,12 @@ int __near_netlink_start_poll(int idx,
 	DBG("IM protos 0x%x TM protos 0x%x", im_protocols, tm_protocols);
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			NLM_F_REQUEST, NFC_CMD_START_POLL, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto nla_put_failure;
 	}
@@ -243,7 +263,6 @@ nla_put_failure:
 	return err;
 }
 
-
 int __near_netlink_stop_poll(int idx)
 {
 	struct nl_msg *msg;
@@ -253,12 +272,12 @@ int __near_netlink_stop_poll(int idx)
 	DBG("");
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			NLM_F_REQUEST, NFC_CMD_STOP_POLL, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto nla_put_failure;
 	}
@@ -285,12 +304,12 @@ int __near_netlink_dep_link_up(uint32_t idx, uint32_t target_idx,
 	DBG("");
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			NLM_F_REQUEST, NFC_CMD_DEP_LINK_UP, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto nla_put_failure;
 	}
@@ -319,12 +338,12 @@ int __near_netlink_dep_link_down(uint32_t idx)
 	DBG("");
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			NLM_F_REQUEST, NFC_CMD_DEP_LINK_DOWN, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto nla_put_failure;
 	}
@@ -341,7 +360,7 @@ nla_put_failure:
 	return err;
 }
 
-int __near_netlink_adapter_enable(int idx, near_bool_t enable)
+int __near_netlink_adapter_enable(int idx, bool enable)
 {
 	struct nl_msg *msg;
 	void *hdr;
@@ -351,17 +370,17 @@ int __near_netlink_adapter_enable(int idx, near_bool_t enable)
 	DBG("");
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
-	if (enable == TRUE)
+	if (enable)
 		cmd = NFC_CMD_DEV_UP;
 	else
 		cmd = NFC_CMD_DEV_DOWN;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			NLM_F_REQUEST, cmd, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto nla_put_failure;
 	}
@@ -386,7 +405,7 @@ static int no_seq_check(struct nl_msg *n, void *arg)
 	return NL_OK;
 }
 
-static int nfc_netlink_event_adapter(struct genlmsghdr *gnlh, near_bool_t add)
+static int nfc_netlink_event_adapter(struct genlmsghdr *gnlh, bool add)
 {
 	struct nlattr *attrs[NFC_ATTR_MAX + 1];
 	uint32_t idx;
@@ -395,29 +414,29 @@ static int nfc_netlink_event_adapter(struct genlmsghdr *gnlh, near_bool_t add)
 
 	nla_parse(attrs, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
-	if (attrs[NFC_ATTR_DEVICE_INDEX] == NULL) {
+	if (!attrs[NFC_ATTR_DEVICE_INDEX]) {
 		near_error("Missing device index");
 		return -ENODEV;
 	}
 
 	idx = nla_get_u32(attrs[NFC_ATTR_DEVICE_INDEX]);
 
-	if (add == TRUE &&
-		(attrs[NFC_ATTR_DEVICE_NAME] == NULL ||
-			attrs[NFC_ATTR_PROTOCOLS] == NULL)) {
+	if (add &&
+		(!attrs[NFC_ATTR_DEVICE_NAME] ||
+			!attrs[NFC_ATTR_PROTOCOLS])) {
 		near_error("Missing attributes");
 		return -EINVAL;
 	}
 
-	if (add == TRUE) {
+	if (add) {
 		char *name;
 		uint32_t protocols;
-		near_bool_t powered;
+		bool powered;
 
 		name = nla_get_string(attrs[NFC_ATTR_DEVICE_NAME]);
 		protocols = nla_get_u32(attrs[NFC_ATTR_PROTOCOLS]);
-		if (attrs[NFC_ATTR_DEVICE_POWERED] == NULL)
-			powered = FALSE;
+		if (!attrs[NFC_ATTR_DEVICE_POWERED])
+			powered = false;
 		else
 			powered = nla_get_u8(attrs[NFC_ATTR_DEVICE_POWERED]);
 
@@ -447,15 +466,15 @@ static int get_targets_handler(struct nl_msg *n, void *arg)
 	target_idx = nla_get_u32(attrs[NFC_ATTR_TARGET_INDEX]);
 	protocols = nla_get_u32(attrs[NFC_ATTR_PROTOCOLS]);
 
-	if (attrs[NFC_ATTR_TARGET_SENS_RES] != NULL)
+	if (attrs[NFC_ATTR_TARGET_SENS_RES])
 		sens_res =
 			nla_get_u16(attrs[NFC_ATTR_TARGET_SENS_RES]);
 
-	if (attrs[NFC_ATTR_TARGET_SEL_RES] != NULL)
+	if (attrs[NFC_ATTR_TARGET_SEL_RES])
 		sel_res =
 			nla_get_u16(attrs[NFC_ATTR_TARGET_SEL_RES]);
 
-	if (attrs[NFC_ATTR_TARGET_NFCID1] != NULL) {
+	if (attrs[NFC_ATTR_TARGET_NFCID1]) {
 		nfcid_len = nla_len(attrs[NFC_ATTR_TARGET_NFCID1]);
 		if (nfcid_len <= NFC_MAX_NFCID1_LEN)
 			memcpy(nfcid, nla_data(attrs[NFC_ATTR_TARGET_NFCID1]),
@@ -473,6 +492,17 @@ static int get_targets_handler(struct nl_msg *n, void *arg)
 	return 0;
 }
 
+static int get_targets_finish_handler(struct nl_msg *n, void *arg)
+{
+	uint32_t adapter_idx;
+
+	DBG("");
+
+	adapter_idx = *((uint32_t *)arg);
+
+	return __near_adapter_get_targets_done(adapter_idx);
+}
+
 static int nfc_netlink_event_targets_found(struct genlmsghdr *gnlh)
 {
 	struct nlattr *attr[NFC_ATTR_MAX + 1];
@@ -485,7 +515,7 @@ static int nfc_netlink_event_targets_found(struct genlmsghdr *gnlh)
 
 	nla_parse(attr, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 			genlmsg_attrlen(gnlh, 0), NULL);
-	if (attr[NFC_ATTR_DEVICE_INDEX] == NULL)
+	if (!attr[NFC_ATTR_DEVICE_INDEX])
 		return -ENODEV;
 
 	adapter_idx = nla_get_u32(attr[NFC_ATTR_DEVICE_INDEX]);
@@ -493,12 +523,12 @@ static int nfc_netlink_event_targets_found(struct genlmsghdr *gnlh)
 	DBG("adapter %d", adapter_idx);
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	hdr = genlmsg_put(msg, NL_AUTO_PID, NL_AUTO_SEQ, nfc_state->nfc_id, 0,
 			NLM_F_DUMP, NFC_CMD_GET_TARGET, NFC_GENL_VERSION);
-	if (hdr == NULL) {
+	if (!hdr) {
 		err = -EINVAL;
 		goto nla_put_failure;
 	}
@@ -507,8 +537,9 @@ static int nfc_netlink_event_targets_found(struct genlmsghdr *gnlh)
 
 	NLA_PUT_U32(msg, NFC_ATTR_DEVICE_INDEX, adapter_idx);
 
-	err = nl_send_msg(nfc_state->cmd_sock, msg,
-				get_targets_handler, &adapter_idx);
+	err = __nl_send_msg(nfc_state->cmd_sock, msg,
+				get_targets_handler, get_targets_finish_handler,
+								&adapter_idx);
 
 nla_put_failure:
 	nlmsg_free(msg);
@@ -526,10 +557,10 @@ static int nfc_netlink_event_target_lost(struct genlmsghdr *gnlh)
 	nla_parse(attr, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 			genlmsg_attrlen(gnlh, 0), NULL);
 
-	if (attr[NFC_ATTR_DEVICE_INDEX] == NULL)
+	if (!attr[NFC_ATTR_DEVICE_INDEX])
 		return -ENODEV;
 
-	if (attr[NFC_ATTR_TARGET_INDEX] == NULL)
+	if (!attr[NFC_ATTR_TARGET_INDEX])
 		return -ENODEV;
 
 	adapter_idx = nla_get_u32(attr[NFC_ATTR_DEVICE_INDEX]);
@@ -550,13 +581,13 @@ static int nfc_netlink_event_dep_up(struct genlmsghdr *gnlh)
 
 	nla_parse(attrs, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
-	if (attrs[NFC_ATTR_DEVICE_INDEX] == NULL) {
+	if (!attrs[NFC_ATTR_DEVICE_INDEX]) {
 		near_error("Missing device index");
 		return -ENODEV;
 	}
 
-	if (attrs[NFC_ATTR_COMM_MODE] == NULL ||
-			attrs[NFC_ATTR_RF_MODE] == NULL) {
+	if (!attrs[NFC_ATTR_COMM_MODE] ||
+			!attrs[NFC_ATTR_RF_MODE]) {
 		near_error("Missing rf or comm modes");
 		return -ENODEV;
 	}
@@ -565,7 +596,7 @@ static int nfc_netlink_event_dep_up(struct genlmsghdr *gnlh)
 	rf_mode = nla_get_u8(attrs[NFC_ATTR_RF_MODE]);
 
 	if (rf_mode == NFC_RF_INITIATOR) {
-		if (attrs[NFC_ATTR_TARGET_INDEX] == NULL) {
+		if (!attrs[NFC_ATTR_TARGET_INDEX]) {
 			near_error("Missing target index");
 			return -ENODEV;
 		};
@@ -574,7 +605,7 @@ static int nfc_netlink_event_dep_up(struct genlmsghdr *gnlh)
 
 		DBG("%d %d", idx, target_idx);
 
-		return __near_adapter_set_dep_state(idx, TRUE);
+		return __near_adapter_set_dep_state(idx, true);
 	} else {
 		return -EOPNOTSUPP;
 	}
@@ -589,14 +620,14 @@ static int nfc_netlink_event_dep_down(struct genlmsghdr *gnlh)
 
 	nla_parse(attrs, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
-	if (attrs[NFC_ATTR_DEVICE_INDEX] == NULL) {
+	if (!attrs[NFC_ATTR_DEVICE_INDEX]) {
 		near_error("Missing device index");
 		return -ENODEV;
 	}
 
 	idx = nla_get_u32(attrs[NFC_ATTR_DEVICE_INDEX]);
 
-	__near_adapter_set_dep_state(idx, FALSE);
+	__near_adapter_set_dep_state(idx, false);
 
 	return 0;
 }
@@ -610,7 +641,7 @@ static int nfc_netlink_event_tm_activated(struct genlmsghdr *gnlh)
 
 	nla_parse(attrs, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
-	if (attrs[NFC_ATTR_DEVICE_INDEX] == NULL) {
+	if (!attrs[NFC_ATTR_DEVICE_INDEX]) {
 		near_error("Missing device index");
 		return -ENODEV;
 	}
@@ -631,7 +662,7 @@ static int nfc_netlink_event_tm_deactivated(struct genlmsghdr *gnlh)
 
 	nla_parse(attrs, NFC_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
-	if (attrs[NFC_ATTR_DEVICE_INDEX] == NULL) {
+	if (!attrs[NFC_ATTR_DEVICE_INDEX]) {
 		near_error("Missing device index");
 		return -ENODEV;
 	}
@@ -645,10 +676,17 @@ static int nfc_netlink_event_tm_deactivated(struct genlmsghdr *gnlh)
 
 static int nfc_netlink_event(struct nl_msg *n, void *arg)
 {
+	struct sockaddr_nl *src = nlmsg_get_src(n);
 	struct genlmsghdr *gnlh = nlmsg_data(nlmsg_hdr(n));
 
 	DBG("event 0x%x", gnlh->cmd);
 
+	if (src->nl_pid) {
+		near_error("WARNING: Wrong netlink message sender %d",
+								src->nl_pid);
+		return NL_SKIP;
+	}
+
 	switch (gnlh->cmd) {
 	case NFC_EVENT_TARGETS_FOUND:
 		DBG("Targets found");
@@ -660,12 +698,12 @@ static int nfc_netlink_event(struct nl_msg *n, void *arg)
 		break;
 	case NFC_EVENT_DEVICE_ADDED:
 		DBG("Adapter added");
-		nfc_netlink_event_adapter(gnlh, TRUE);
+		nfc_netlink_event_adapter(gnlh, true);
 
 		break;
 	case NFC_EVENT_DEVICE_REMOVED:
 		DBG("Adapter removed");
-		nfc_netlink_event_adapter(gnlh, FALSE);
+		nfc_netlink_event_adapter(gnlh, false);
 
 		break;
 	case NFC_CMD_DEP_LINK_UP:
@@ -703,7 +741,7 @@ static gboolean __nfc_netlink_event(GIOChannel *channel,
 		return FALSE;
 
 	cb = nl_cb_alloc(NL_CB_VERBOSE);
-	if (cb == NULL)
+	if (!cb)
 		return TRUE;
 
 	nl_cb_set(cb, NL_CB_SEQ_CHECK, NL_CB_CUSTOM, no_seq_check, NULL);
@@ -752,7 +790,7 @@ static int family_handler(struct nl_msg *msg, void *arg)
 	nla_parse(tb, CTRL_ATTR_MAX, genlmsg_attrdata(gnlh, 0),
 		  genlmsg_attrlen(gnlh, 0), NULL);
 
-        if (!tb[CTRL_ATTR_MCAST_GROUPS])
+	if (!tb[CTRL_ATTR_MCAST_GROUPS])
 		return NL_SKIP;
 
 	nla_for_each_nested(mcgrp, tb[CTRL_ATTR_MCAST_GROUPS], rem_mcgrp) {
@@ -770,7 +808,7 @@ static int family_handler(struct nl_msg *msg, void *arg)
 		grp->id = nla_get_u32(tb_mcgrp[CTRL_ATTR_MCAST_GRP_ID]);
 		break;
 	}
-	
+
 	return NL_SKIP;
 }
 
@@ -787,13 +825,12 @@ static int nl_get_multicast_id(struct nl_sock *sock, const char *family,
 	DBG("");
 
 	msg = nlmsg_alloc();
-	if (msg == NULL)
+	if (!msg)
 		return -ENOMEM;
 
 	ctrlid = genl_ctrl_resolve(sock, "nlctrl");
 
-        genlmsg_put(msg, 0, 0, ctrlid, 0,
-		    0, CTRL_CMD_GETFAMILY, 0);
+	genlmsg_put(msg, 0, 0, ctrlid, 0, 0, CTRL_CMD_GETFAMILY, 0);
 
 	err = -EMSGSIZE;
 
@@ -820,18 +857,18 @@ int __near_netlink_init(void)
 	DBG("");
 
 	nfc_state = g_try_malloc0(sizeof(struct nlnfc_state));
-	if (nfc_state == NULL)
+	if (!nfc_state)
 		return -ENOMEM;
 
 	nfc_state->cmd_sock = nl_socket_alloc();
-	if (nfc_state->cmd_sock == NULL) {
+	if (!nfc_state->cmd_sock) {
 		near_error("Failed to allocate NFC command netlink socket");
 		err = -ENOMEM;
 		goto state_free;
 	}
 
 	nfc_state->event_sock = nl_socket_alloc();
-	if (nfc_state->event_sock == NULL) {
+	if (!nfc_state->event_sock) {
 		near_error("Failed to allocate NFC event netlink socket");
 		err = -ENOMEM;
 		goto handle_cmd_destroy;
@@ -890,14 +927,14 @@ state_free:
 
 void __near_netlink_cleanup(void)
 {
-	if (netlink_channel != NULL) {
+	if (netlink_channel) {
 		g_io_channel_shutdown(netlink_channel, TRUE, NULL);
 		g_io_channel_unref(netlink_channel);
 
 		netlink_channel = NULL;
 	}
 
-	if (nfc_state == NULL)
+	if (!nfc_state)
 		return;
 
 	nl_socket_free(nfc_state->cmd_sock);
@@ -907,4 +944,3 @@ void __near_netlink_cleanup(void)
 
 	DBG("");
 }
-
